import AuthService from "../services/auth.service";
import AuthStore from "./auth.store";
import PermissionStackUtil from './permission.stack.handler';
const axios = require('axios');

/**
 * Global functions to authenticate user and get user details from browser local storage.
 */
const Auth = {
    /** To check if user already logged in 
     * - First it will check the API token is available in the browser local storage
     * - If not then check the Keycloak authentication
     * - Return false if no authentication found
     */
    isAuthenticated: async () => {
        let user = localStorage.getItem("user");
        if (user) {
            user = JSON.parse(user);
            return Auth.getUserPermission(user);
        }   else {
            //Keycloak authentication
            const res = await AuthService.getKeycloakAuthState();
            if (res && res.is_authenticated) {
                localStorage.setItem("loginType", 'Keycloak');
                const cookies = document.cookie.split(';').reduce((prev, current) => {
                    const [name, value] = current.split(/\s?(.*?)=(.*)/).splice(1, 2);
                    prev[name] = value;
                    return prev;
                }, {});
                const userDetails = {'name':res.username, 'token': cookies.csrftoken};
                // axios.defaults.headers.common['Authorization'] = `Token ${cookies.csrftoken}`;
                localStorage.setItem("user", JSON.stringify(userDetails));
                return Auth.getUserPermission(userDetails);
            }
        }
        await PermissionStackUtil.getPermissions(false);
        return false;
    },
    getUserPermission: async (user) => {
        if (user.token) {
            axios.defaults.headers.common['Authorization'] = `Token ${user.token}`;
            axios.defaults.headers.common['X-CSRFTOKEN'] = `${user.token}`;
            const permissions = await AuthStore.getState();
            if(!permissions.userRolePermission.project) {
                await PermissionStackUtil.getPermissions(true);
            }
            return true;
        }
    },
    /** Gets user details from browser local storage */
    getUser: () => {
        return JSON.parse(localStorage.getItem("user"));
    },
    /** Authenticate user from the backend and store user details in local storage */
    login: async(user, pass) => {
        const authData = await AuthService.authenticate(user, pass);
        if (authData) {
            localStorage.setItem("user", JSON.stringify({name:user, token: authData.token},));
            localStorage.setItem("loginType", 'Application');
            await PermissionStackUtil.getPermissions(true);
            return true;
        }   else {
            return false;
        }
    },
    /** Remove user details from localstorage on logout */
    logout: () => {
        AuthService.deAuthenticate();
        PermissionStackUtil.deleteStore()
        localStorage.removeItem("user");
    }
}

export default Auth;